//
// IrrKlang Wrapper for Imperative Languages
// Frank Dodd (2006-2009)
//
// This wrapper DLL encapsulates a sub-set of the features of the powerful
// IrrKlang 3D Audio Engine exposing the Object Oriented architecture and
// providing a functional Audio SDK for languages that are not object oriented.
//
// This software was developed using the GCC compiler and Code::Blocks IDE
//

/* ////////////////////////////////////////////////////////////////////////////
includes
*/

#include <stdio.h>
#include "irrklangwrapper.h"


/* ////////////////////////////////////////////////////////////////////////////
global variables
*/

/* ////////////////////////////////////////////////////////////////////////////
external variables
*/
extern ISoundEngine* engine;


/* ////////////////////////////////////////////////////////////////////////////
Global Function Declarations

all of the below functions are declared as C functions and are exposed without
any mangled names so that they can be easily imported into imperative
languages like FreeBasic
*/
extern "C"
{

/* ////////////////////////////////////////////////////////////////////////////
SYSTEM FUNCTIONS
*/
/* ----------------------------------------------------------------------------
Get the length of the sound in milliseconds
*/
unsigned int DLL_EXPORT IrrKlangGetPlayLength( ISound * sound )
{
    return sound->getPlayLength();
}

/* ----------------------------------------------------------------------------
Get the position of play in the sound in milliseconds
*/
unsigned int DLL_EXPORT IrrKlangGetPlayPosition( ISound * sound )
{
    return sound->getPlayPosition();
}

/* ----------------------------------------------------------------------------
Get the position of play in the sound in milliseconds
*/
unsigned int DLL_EXPORT IrrKlangIsFinished( ISound * sound )
{
    return sound->isFinished();
}

/* ----------------------------------------------------------------------------
Pause and unpause the sound
*/
void DLL_EXPORT IrrKlangSetIsPaused( ISound * sound, unsigned int pause )
{
    sound->setIsPaused( pause );
}

/* ----------------------------------------------------------------------------
Set the playback speed of the sound
*/
unsigned int DLL_EXPORT IrrKlangSetPlaybackSpeed( ISound * sound, float speed )
{
    return sound->setPlaybackSpeed( speed );
}

/* ----------------------------------------------------------------------------
Set the playback position of the sound
*/
unsigned int DLL_EXPORT IrrKlangSetPlayPosition( ISound * sound, unsigned int position )
{
    return sound->setPlayPosition( position );
}

/* ----------------------------------------------------------------------------
Set the position of the sound in 3D space
*/
void DLL_EXPORT IrrKlangSetPosition( ISound * sound, float posX, float posY, float posZ )
{
    sound->setPosition( vec3df( posX, posY, posZ ));
}

/* ----------------------------------------------------------------------------
Set the volume of the sound
*/
void DLL_EXPORT IrrKlangSetVolume( ISound * sound, float volume )
{
    sound->setVolume( volume );
}

/* ----------------------------------------------------------------------------
Set the minimum distance at which the sound can be heard
*/
void DLL_EXPORT IrrKlangSetMinDistance( ISound * sound, float distance )
{
    sound->setMinDistance( distance );
}

/* ----------------------------------------------------------------------------
Stop the sound playing
*/
void DLL_EXPORT IrrKlangStopSound( ISound * sound )
{
    sound->stop();
}

//returns if the sound is paused
unsigned int DLL_EXPORT IrrKlangGetIsPaused( ISound * sound )
{
   return sound->getIsPaused();
}

//returns volume of the sound
float DLL_EXPORT IrrKlangGetVolume( ISound * sound )
{
   return sound->getVolume();
}

//sets the pan of the sound
void DLL_EXPORT IrrKlangSetPan( ISound * sound, float pan )
{
   return sound->setPan(pan);
}

//returns the pan of the sound
float DLL_EXPORT IrrKlangGetPan( ISound * sound )
{
   return sound->getPan();
}

//returns if the sound has been started to play looped
unsigned int DLL_EXPORT IrrKlangIsLooped( ISound * sound )
{
   return sound->isLooped();
}

// changes the loop mode of the sound
void DLL_EXPORT IrrKlangSetIsLooped( ISound * sound, unsigned int looped )
{
  return sound->setIsLooped( looped );
}

//Returns the minimal distance if this is a 3D sound
float DLL_EXPORT IrrKlangGetMinDistance( ISound * sound )
{
   return sound->getMinDistance();
}

//Sets the maximal distance if this is a 3D sound
void DLL_EXPORT IrrKlangSetMaxDistance( ISound * sound, float distance )
{
   return sound->setMaxDistance( distance );
}

//Returns the maximal distance if this is a 3D sound
float DLL_EXPORT IrrKlangGetMaxDistance( ISound * sound )
{
  return sound->getMaxDistance();
}

//Returns the playback speed set by setPlaybackSpeed()
float DLL_EXPORT IrrKlangGetPlaybackSpeed( ISound * sound )
{
  return sound->getPlaybackSpeed();
}


/* ////////////////////////////////////////////////////////////////////////////
all of the above functions are declared as C functions and are exposed without
any mangled names
*/
}
